;ia_pni.inc MASM Macro definitions for Streaming SIMD Extensions 3
;THIS SOFTWARE AND DOCUMENTATION IS PROVIDED "AS IS" WITH NO WARRANTIES
;WHATSOEVER, INCLUDING ANY WARRANTY OF MERCHANTABILITY, NON-INFRINGEMENT,
;FITNESS FOR ANY PARTICULAR PURPOSE, OR ANY WARRANTY OTHERWISE ARISING
;OUT OF ANY PROPOSAL, SPECIFICATION OR SAMPLE.

;Intel disclaims all liability, including liability for infringement
;of any proprietary rights, relating to use of information in this
;software. No license, express or implied, by estoppel or otherwise,
;to any intellectual property rights is granted herein. Intel retains
;the right to make changes to its software and documentation at any time,
;without notice.

;The software vendor remains solely responsible for the design, sale,
;and functionality of its product, including any liability arising
;from product infringement or product warranty of any kind.

;Copyright (c) 2003 Intel Corporation.
;All rights reserved.


.686P
.xmm

;This macro package requires an assembler vesion 6.15.8803 or later.

;Please use XMMWORD and not DWORD (OWORD does not work) for 128 bit data
;in Streaming SIMD Extensions 2 instructions. After getting a real
;assembler you will just have to add the line "XMMWORD TEXTEQU <OWORD>"
;to your code. 

  MM2WORD TEXTEQU <QWORD> ; used only by the compiler, obsolete
MMWORD TEXTEQU <QWORD> ; used only by the compiler, obsolete
XMMWORD TEXTEQU <DWORD> ; 128 bit memory operands for xmm regs inst

opc_addsubpd = 0D0H
opc_addsubps = 0D0H
opc_fisttp16 = 0DFH
opc_fisttp32 = 0DBH
opc_fisttp64 = 0DDH
opc_lddqu    = 0F0H
opc_movddup  = 012H
opc_movshdup = 016H
opc_movsldup = 012H
opc_haddps   = 07CH
opc_haddpd   = 07CH
opc_hsubps   = 07DH
opc_hsubpd   = 07DH

pref_66                = 066H
pref_f2                = 0F2H
pref_f3                = 0F3H


emm_inst macro pref:req, op:req, dst:req, src:req
     local x,y
x:
     addpd          dst, src
y:
     org x
     byte pref
     org x+2
     byte op
     org y
endm

emm_66_op     macro     op:req,     dst:req,     src:req
     emm_inst     pref_66,   op, dst, src
endm

emm_f2_op     macro     op:req,     dst:req,     src:req
     emm_inst     pref_f2,   op, dst, src
endm

emm_f3_op    macro      op:req,     dst:req,     src:req
     emm_inst     pref_f3,   op, dst, src
endm
 
; 66 0F D0 /r        addsubpd       xmm1,        xmm2/m128
 addsubpd    macro     dst:req,     src:req
  emm_66_op       opc_addsubpd,     dst, src
endm

; F2 0F D0 /r addsubps xmm1, xmm2/m64
addsubps     macro     dst:req,     src:req
 emm_f2_op        opc_addsubps,     dst, src
endm

; F2 0F F0 /r lddqu xmm, m128
lddqu       macro      dst:req,     src:req
IF (OPATTR(src) AND 00010100y) ; register or constant
   .ERR <illegal operands. dst should be memory !>
ELSE
    emm_f2_op opc_lddqu, dst, src
ENDIF
endm

; F2 0F 12 /r movddup xmm1, xmm2/m64
movddup     macro     dst:req,      src:req
   emm_f2_op opc_movddup, dst, src
endm

; F3 0F 16 /r movshdup xmm1, xmm2/m128
movshdup   macro      dst:req,      src:req
  emm_f3_op opc_movshdup, dst, src
endm

; F3 0F 12 /r movsldup xmm1, xmm2/m128
movsldup   macro      dst:req,      src:req
  emm_f3_op opc_movsldup, dst, src
endm

; F2 0F 7C /r haddps xmm1, xmm2/m64
haddps     macro      dst:req,      src:req
   emm_f2_op opc_haddps, dst, src
endm

; 66 0F 7C /r haddpd xmm1, xmm2/m128
haddpd     macro     dst:req,       src:req
   emm_66_op opc_haddpd, dst, src
endm

; F2 0F 7D /r hsubps xmm1, xmm2/m64
hsubps     macro     dst:req,       src:req
   emm_f2_op opc_hsubps, dst, src
endm

; 66 0F 7D /r hsubpd xmm1, xmm2/m128
hsubpd     macro     dst:req,       src:req
   emm_66_op opc_hsubpd, dst, src
endm

; DF /1               fisttp m16int
; DB /1               fisttp m32int
; DD /1               fisttp m64int
fisttp     macro     adr:req
           local x, y
IF (OPATTR(adr)) AND 00010100y ; register or const
       .ERR <invalid operand. dst must be an address in memory!>
ELSEIF (TYPE (adr) EQ WORD) OR (TYPE (adr) EQ SWORD)
x:
        fimul    adr
y:
        org      x
        byte     opc_fisttp16
        org      y
ELSEIF (TYPE (adr) EQ DWORD) OR (TYPE (adr) EQ SDWORD)
        x:
          fimul adr
        y:
          org    x
          byte   opc_fisttp32
          org    y
      ELSEIF (TYPE (adr) EQ QWORD)
        x:
          fmul   adr
        y:
          org    x
          byte   opc_fisttp64
          org    y
      ELSE
              .ERR <invalid operand. dst can be 2,4 or 8 bytes address only!>
      ENDIF
endm

; 0F 01 C8 monitor
monitor macro
    byte 0x0f
    byte 0x01
    byte 0xc8
endm

; 0F 01 C8 mwait
mwait macro
    byte 0x0f
    byte 0x01
    byte 0xc9
endm 
